

var PitchButtonGroup = function()
{
    SensorGroup.prototype.constructor.call( this );

    this.sensors.octave   = new RangedSensorPair( 0, -4, 4, 'octaveUp', 'octaveDown', true );
    this.sensors.semitone = new RangedSensorPair( 0, -12, 12, 'semitoneUp', 'semitoneDown', true );
    this.sensors.scale    = new CircleSensorPair( 0, app.scales.length, 'scaleUp', 'scaleDown' );

    this.addListener( 'octave', 'sensorOn', this, function( sensor ) {
        app.notify( 'Octave: ' + sensor.value );
        app.preferences.octaveSetting.setRaw( sensor.value );
    });
    this.addListener( 'semitone', 'sensorOn', this, function( sensor ) {
        app.notify( 'Root: ' + MidiData.getRootName( sensor.value ));
        app.preferences.semitoneSetting.setRaw( sensor.value );
    });
    this.addListener( 'scale', 'sensorOn', this, function( sensor ) {
        app.preferences.scaleSetting.setRaw( sensor.value );
        var rootNote = MidiData.getRootName( this.sensors.semitone.value );
        var scaleName = rootNote + ' ' + app.scales[sensor.value].name;
        app.notify( 'Scale: ' + scaleName );
        this.update();
    });

    this.listenTo( app.preferences, 'octave',   'onPreferences' );
    this.listenTo( app.preferences, 'semitone', 'onPreferences' );
    this.listenTo( app.preferences, 'scale',    'onPreferences' );
};
PitchButtonGroup.prototype = new SensorGroup();
PitchButtonGroup.prototype.constructor = PitchButtonGroup;


PitchButtonGroup.prototype.update = function() {
    if( app.getMode() !== 'clips' ) SensorGroup.prototype.update.call( this );
};


PitchButtonGroup.prototype.onNoteOn = function( midi )
{
    switch( midi.data1 ) {
        case app.preset.octaveUp.note     : this.sensors.octave.increase(); break;
        case app.preset.octaveDown.note   : this.sensors.octave.decrease(); break;
        case app.preset.semitoneUp.note   : this.sensors.semitone.increase(); break;
        case app.preset.semitoneDown.note : this.sensors.semitone.decrease(); break;
        case app.preset.scaleUp.note      : this.sensors.scale.increase(); break;
        case app.preset.scaleDown.note    : this.sensors.scale.decrease(); break;
    }
};

PitchButtonGroup.prototype.onNoteOff = function( midi )
{
    switch( midi.data1 ) {
        case app.preset.octaveUp.note     :
        case app.preset.octaveDown.note   : this.sensors.octave.onRelease(); break;
        case app.preset.semitoneUp.note   :
        case app.preset.semitoneDown.note : this.sensors.semitone.onRelease(); break;
        case app.preset.scaleUp.note      :
        case app.preset.scaleDown.note    : this.sensors.scale.onRelease(); break;
    }
};
