
/**
 * Provides a layer between quneopresets and the midi mapping used by this application.
 * The presets defined in PresetMapper maps the midi code send by each sensor in a specific preset to generic sensor names.
 */
var Presets = function() {};


Presets.prototype.init = function()
{
    this.loadPreset();

    app.inPort.setSysexCallback( function() {
        if( sysex.matchesHexPattern( 'f07e00060300015f1e0000001e1200??f7' )) {    // Quneo load pattern
            app.sensors.update();
        }
    });
};


Presets.prototype.loadPreset = function()
{
    this.preset = {};
    for( var key in Presets.nameMap )
    {
        var sensorName             = Presets.nameMap[key];
        this.preset[key]           = map[sensorName];
        this.preset[key].ledNum    = Presets.ledMap[sensorName].ledNum;
        this.preset[key].ledStatus = Presets.ledMap[sensorName].ledStatus;
    }
    app.sensors.update();
};


/**
 * Maps the functions of this application to Quneo buttons
 */
Presets.nameMap =
{
    rootNote        : 'pad1',
    transportStart  : 'playButton',
    transportStop   : 'stopButton',
    transportRecord : 'recordButton',

    octaveUp        : 'upButton1',
    octaveDown      : 'downButton1',
    semitoneUp      : 'upButton2',
    semitoneDown    : 'downButton2',

    trackDown       : 'leftButton1',
    trackUp         : 'rightButton1',
    targetParam     : 'leftButton2',
    targetMacro     : 'rightButton2',
    paramsDown      : 'leftButton3',
    paramsUp        : 'rightButton3',
    scaleDown       : 'leftButton4',
    scaleUp         : 'rightButton4',

    setMode         : 'rhombusButton',
    trackVolume     : 'rotary1',
    masterVolume    : 'rotary2',
    pitchBend       : 'longSlider',

    slider1         : 'vslider1',
    slider2         : 'vslider2',
    slider3         : 'vslider3',
    slider4         : 'vslider4',
    slider5         : 'hslider1',
    slider6         : 'hslider2',
    slider7         : 'hslider3',
    slider8         : 'hslider4'
};

/**
 * Maps the Quneo buttons to their led codes.
 */
Presets.ledMap = {
    pad1         : { name: 'pad1',          note: 36, ledNum: 0,  ledStatus: 144 },
    playButton   : { name: 'playButton',    note: 31, ledNum: 35, ledStatus: 144 },
    stopButton   : { name: 'stopButton',    note: 30, ledNum: 34, ledStatus: 144 },
    recordButton : { name: 'recordButton',  note: 29, ledNum: 33, ledStatus: 144 },

    upButton1    : { name: 'upButton1',     note: 20, ledNum: 46, ledStatus: 144 },
    downButton1  : { name: 'downButton1',   note: 21, ledNum: 47, ledStatus: 144 },
    upButton2    : { name: 'upButton2',     note: 22, ledNum: 48, ledStatus: 144 },
    downButton2  : { name: 'downButton2',   note: 23, ledNum: 49, ledStatus: 144 },

    leftButton1  : { name: 'leftButton1',   note: 23, ledNum: 36, ledStatus: 144 },
    rightButton1 : { name: 'rightButton1',  note: 11, ledNum: 37, ledStatus: 144 },
    leftButton2  : { name: 'leftButton2',   note: 12, ledNum: 38, ledStatus: 144 },
    rightButton2 : { name: 'rightButton2',  note: 13, ledNum: 39, ledStatus: 144 },
    leftButton3  : { name: 'leftButton2',   note: 14, ledNum: 40, ledStatus: 144 },
    rightButton3 : { name: 'rightButton3',  note: 15, ledNum: 41, ledStatus: 144 },
    leftButton4  : { name: 'leftButton4',   note: 16, ledNum: 42, ledStatus: 144 },
    rightButton4 : { name: 'rightButton4',  note: 17, ledNum: 43, ledStatus: 144 },

    rhombusButton: { name: 'rhombusButton', note: 32,        ledNum: 44, ledStatus: 144 },
    longSlider   : { name: 'longSlider',    note: 4, cc: 11, ledNum: 5,  ledStatus: 176 },
    rotary1      : { name: 'rotary1',       note: 5, cc: 12, ledNum: 6,  ledStatus: 176 },
    rotary2      : { name: 'rotary2',       note: 0, cc: 10, ledNum: 7,  ledStatus: 176 },

    vslider1     : { name: 'vslider1',      note: 1, cc: 1,  ledNum: 1,  ledStatus: 176 },
    vslider2     : { name: 'vslider2',      note: 2, cc: 2,  ledNum: 2,  ledStatus: 176 },
    vslider3     : { name: 'vslider3',      note: 3, cc: 3,  ledNum: 3,  ledStatus: 176 },
    vslider4     : { name: 'vslider4',      note: 4, cc: 4,  ledNum: 4,  ledStatus: 176 },
    hslider1     : { name: 'hslider1',      note: 5, cc: 5,  ledNum: 11, ledStatus: 176 },
    hslider2     : { name: 'hslider2',      note: 6, cc: 6,  ledNum: 10, ledStatus: 176 },
    hslider3     : { name: 'hslider3',      note: 7, cc: 7,  ledNum: 9,  ledStatus: 176 },
    hslider4     : { name: 'hslider4',      note: 8, cc: 8,  ledNum: 8,  ledStatus: 176 },
};




Presets.prototype.presetSysex = [
    'F0 00 01 5F 7A 1E 00 01 00 02 30 00 6F 35 00 10 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 01 7F 14 00 10 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 02 4F 77 00 10 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 03 5F 56 00 10 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 04 2F 31 00 30 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 05 3F 10 00 30 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 06 0F 73 00 30 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 07 1F 52 00 30 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 08 6E 3D 00 00 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 09 7E 1C 00 00 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0A 4E 7F 00 00 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0B 5E 5E 00 00 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0C 2E 39 00 20 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0D 3E 18 00 20 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0E 0E 7B 00 20 10 F7',
    'F0 00 01 5F 7A 1E 00 01 00 02 30 0F 1E 5A 00 20 10 F7',
];
