/**
 * ClipButtonGroup
 * A button group that controls the clip launcher.
 * @constructor
 * @author 3ears
 * @mail tom-buettner@t-online.de
 */
var ClipButtonGroup = function()
{
    this.sensors.scrollChannels = new AutomatedSensorPair( 'clipsUp', 'clipsDown' );
    this.sensors.scrollScenes   = new AutomatedSensorPair( 'clipsRight', 'clipsLeft' );

    this.addListener( 'scrollChannels', 'sensorInc', this, function() {
        app.clipLauncher.scrollChannelsUp();
    });
    this.addListener( 'scrollChannels', 'sensorDec', this, function() {
        app.clipLauncher.scrollChannelsDown();
    });
    this.addListener( 'scrollScenes', 'sensorDec', this, function() {
        app.clipLauncher.scrollScenesUp();
    });
    this.addListener( 'scrollScenes', 'sensorInc', this, function() {
        app.clipLauncher.scrollScenesDown();
    });

    var self = this;
    app.clipLauncher.addCanScrollChannelsUpObserver( function( canScroll ) {
        self.sensors.scrollChannels.enableIncrease( canScroll, self.isActive() );
    });
    app.clipLauncher.addCanScrollChannelsDownObserver( function( canScroll ) {
        self.sensors.scrollChannels.enableDecrease( canScroll, self.isActive() );
    });
    app.clipLauncher.addCanScrollScenesDownObserver( function( canScroll ) {
        self.sensors.scrollScenes.enableIncrease( canScroll, self.isActive() );
    });
    app.clipLauncher.addCanScrollScenesUpObserver( function( canScroll ) {
        self.sensors.scrollScenes.enableDecrease( canScroll, self.isActive() );
    });

    app.clipLauncher.addTrackScrollPositionObserver( function( pos ) {
        app.notify( 'Track Scroll: ' + pos );
    }, 1 );
    app.clipLauncher.addSceneScrollPositionObserver( function( pos ) {
        app.notify( 'Scene Scroll: ' + pos );
    }, 1 );

    app.preferences.addListener( 'mode', this, this.updateLauncherIndication );
    app.preferences.addListener( 'splitMode', this, this.updateLauncherIndication );
};
ClipButtonGroup.prototype = new SensorGroup();
ClipButtonGroup.prototype.constructor = ClipButtonGroup;


ClipButtonGroup.prototype.updateLeds = function() {
    if( this.isActive() ) SensorGroup.prototype.updateLeds.call( this );
};


/**
 * Update the rectangle in clip launcher that show the section covered by the pads.
 */
ClipButtonGroup.prototype.updateLauncherIndication = function()
{
    var mode = app.getMainMode();
    var activeChannels = mode === 'split' ? 2 : 4;

    for( var i = 0; i < 4; i++ )
    {
        var channel = app.clipLauncher.getChannel( i );
        var slots   = channel.getClipLauncherSlots();
        var active  = !!(i < activeChannels && mode !== 'notes');

        slots.setIndication( active );
    }
};


ClipButtonGroup.prototype.isActive = function() {
    return !!(app.getMode() === 'clips' || app.getMode() === 'alternate');
};


ClipButtonGroup.prototype.onNoteOn = function( midi )
{
    if( this.isActive() ) {
        switch( midi.data1 ) {
            case app.layout.clipsUp.note    : this.sensors.scrollChannels.increase(); break;
            case app.layout.clipsDown.note  : this.sensors.scrollChannels.decrease(); break;
            case app.layout.clipsLeft.note  : this.sensors.scrollScenes.decrease(); break;
            case app.layout.clipsRight.note : this.sensors.scrollScenes.increase(); break;
        }
    }
};


ClipButtonGroup.prototype.onNoteOff = function( midi )
{
    if( this.isActive() ) {
        switch( midi.data1 ) {
            case app.layout.clipsUp.note    :
            case app.layout.clipsDown.note  : this.sensors.scrollChannels.onRelease(); break;
            case app.layout.clipsLeft.note  :
            case app.layout.clipsRight.note : this.sensors.scrollScenes.onRelease(); break;
        }
    }
};





