
/**
 * MidiData
 * Helper class to access midi data
 * @param status
 * @param data1
 * @param data2
 *
 * @author 3ears
 * @mail tom-buettner@t-online.de
 */
var MidiData = function( status, data1, data2 )
{
    this.status = status;
    this.data1  = data1;
    this.data2  = data2;
};


MidiData.noteNames = [
    'C-2','C-2#','D-2','D-2#','E-2','F-2','F-2#','G-2','G-2#','A-2','A-2#','B-2',
    'C-1','C-1#','D-1','D-1#','E-1','F-1','F-1#','G-1','G-1#','A-1','A-1#','B-1',
    'C0','C0#','D0','D0#','E0','F0','F0#','G0','G0#','A0','A0#','B0',
    'C1','C1#','D1','D1#','E1','F1','F1#','G1','G1#','A1','A1#','B1',
    'C2','C2#','D2','D2#','E2','F2','F2#','G2','G2#','A2','A2#','B2',
    'C3','C3#','D3','D3#','E3','F3','F3#','G3','G3#','A3','A3#','B3',
    'C4','C4#','D4','D4#','E4','F4','F4#','G4','G4#','A4','A4#','B4',
    'C5','C5#','D5','D5#','E5','F5','F5#','G5','G5#','A5','A5#','B5',
    'C6','C6#','D6','D6#','E6','F6','F6#','G6','G6#','A6','A6#','B6',
    'C7','C7#','D7','D7#','E7','F7','F7#','G7','G7#','A7','A7#','B7',
    'C8','C8#','D8','D8#','E8','F8','F8#','G8'
];


MidiData.rootNames = [
    'C', 'C#', 'D', 'D#/Eb', 'E', 'F', 'F#/Gb', 'G', 'G#/Ab', 'A', 'A#/Bb', 'B'
];


MidiData.getRootName = function( noteNum )
{
    var length = MidiData.rootNames.length;
    noteNum = noteNum % length;
    if( noteNum < 0 ) {
        noteNum = length - Math.abs( noteNum );
    }
    return MidiData.rootNames[noteNum];
};


MidiData.encode14BitValue = function( value )
{
    var mask = 0x007F;
    return {
        lsb: value & mask,
        msb: (value & (mask << 7)) >> 7,
    };
};


MidiData.prototype =
{
    channel: function() { return (this.status & 0xF); },

    isNoteOff: function() {
        return ((this.status & 0xF0) === 0x80) ||
               ((this.status & 0xF0) === 0x90 && this.data2 === 0);
    },
    isNoteOn: 				function() { return (this.status & 0xF0) === 0x90; },
    isNote:                 function() { return this.isNoteOn() || this.isNoteOff(); },
    isKeyPressure: 			function() { return (this.status & 0xF0) === 0xA0; },
    isController: 	        function() { return (this.status & 0xF0) === 0xB0; },
    isProgramChange: 		function() { return (this.status & 0xF0) === 0xC0; },
    isChannelPressure: 		function() { return (this.status & 0xF0) === 0xD0; },
    isPitchBend: 			function() { return (this.status & 0xF0) === 0xE0; },
    isMTCQuarterFrame: 		function() { return (this.status === 0xF1); },
    isSongPositionPointer: 	function() { return (this.status === 0xF2); },
    isSongSelect: 			function() { return (this.status === 0xF3); },
    isTuneRequest: 			function() { return (this.status === 0xF6); },
    isTimingClock: 			function() { return (this.status === 0xF8); },
    isMIDIStart: 			function() { return (this.status === 0xFA); },
    isMIDIContinue: 		function() { return (this.status === 0xFB); },
    isMIDIStop: 			function() { return (this.status === 0xFC); },
    isActiveSensing: 		function() { return (this.status === 0xFE); },
    isSystemReset: 			function() { return (this.status === 0xFF); },

    type: function() { return this.status & 0xF0; },

    typeString: function()
    {
        switch( this.type() ) {
            case 0x80: return 'NoteOff';
            case 0x90: return this.data1 === 0 ? 'NoteOff' : 'NoteOn';
            case 0xA0: return 'KeyPressure';
            case 0xB0: return 'CC';
            case 0xC0: return 'ProgramChange';
            case 0xD0: return 'ChannelPressure';
            case 0xE0: return 'PitchBend';
        };
        return 'Other';
    },

    toString: function() {
        return uint8ToHex( this.status ) + uint7ToHex( this.data1 ) + uint7ToHex( this.data2 );
    },
};


